#!/usr/bin/env python
# encoding: utf-8
"""
Util.py

Created by Francisco Gray <frgray@uoregon.edu> on 2012-11-01.
Copyright (c) 2012 University of Oregon. All rights reserved.
DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
"""

import os
import sys
import re
import xml.etree.ElementTree as ET
import xmltodict


class NetdotError(Exception):
    pass


def filter_dict(dict, kword):
    """
    This function descends into the Multi-level
    dictionary and returns a list of [filtered] key value pairs

    Usage:
      dot.filter_dict(dict, ['list', 'of', '.*keywords'])

    Returns:
      Multi-level dictionary on success
    """
    data = {}
    for top_k, top_v in dict.items():
        data[top_k] = {}
        for mid_k, mid_v in top_v.items():
            data[top_k][mid_k] = {}
            for bot_k, bot_v in mid_v.items():
                if kword:
                    re_comb = "(" + ")|(".join(kword) + ")"
                    if re.match(re_comb, bot_k):
                        data[top_k][mid_k][bot_k] = bot_v
                    else:
                        data[top_k][mid_k][bot_k] = bot_v
    return data


def validate_xml(content):
    if '<opt' not in str(content):
        raise NetdotError(content)


def parse_xml(xml):
    """
    This is a VERY simple parser specifically built to
    parse the NetDot-XML Objects

    Returns:
      Multi-level dictionary.
    """
    validate_xml(xml)
    data = {}
    xml_root = ET.fromstring(xml)
    if xml_root.attrib:
        # root has attributes, so we're likely
        # receiving a single object
        data = xml_root.attrib
    else:
        # No root attributes means that we're
        # receiving a list of objects
        for child in xml_root:
            if child.tag in data:
                data[child.tag][child.attrib["id"]] = child.attrib
            else:
                data[child.tag] = {}
                data[child.tag][child.attrib["id"]] = child.attrib

    return data


def parse_xml2(xml):
    return xmltodict.parse(xml, attr_prefix='')


def dump(object):
    for property, value in vars(object).items():
        print(property, ": ", value)



from distutils.core import setup
setup(
    name = 'uo_vra_netdot_client',         # How you named your package folder (MyLib)
    packages = ['uo_vra_netdot_client'],   # Chose the same as "name"
    version = '0.4',      # Start with a small number and increase it with every change you make
    license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description = 'Package for connecting VRealize Automation to Netdot for IPAM',   # Give a short description about your library
    author = 'University of Oregon',                   # Type in your name
    author_email = 'systems@uoregon.edu',      # Type in your E-Mail
    url = 'https://git.uoregon.edu/projects/ISV/repos/uo_vra_netdot_client/browse',   # Provide either the link to your github or to your website
    # download_url = 'NOT SURE YET',    # I explain this later on
    keywords = ['IPAM', 'VRA', 'NETDOT'],   # Keywords that define your package best
    install_requires=[            # I get to this in a second
            'beautifulsoup4',
        ],
    classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    ],
)
